/*
 * 
 */
package org.jhotdraw.cmdcontracts;

import org.jhotdraw.standard.AbstractCommand;
import org.jhotdraw.framework.JHotDrawRuntimeException;

/**
 * The execute() methods in all the concrete, non-anonymous, 
 * non-abstract command classes that extend AbstractCommand, 
 * will first check the reference to the active view for non-null
 * values. This is a contract enforcement concern. 
 * (Originally this was implemented as a call to the 
 * super's functionality - super.execute() - that checked the reference.
 * 
 * @author Marius M.
 *   
 */


public aspect CmdCheckViewRef {

    /**
	 * The poincut that captures the execution of all
	 * execute() methods in the non-anonymous subclasses of
	 * AbstractCommand, and AbstractCommand itself.
	 * NOTE: There is an AspectJ poincut definition limitation
	 * that does not allow to consistently exclude the anonymous
	 * classes form the Command hierarchy specification of the 
	 * pointcut. For this reason, naming convention and code info
	 * (such as observing the classes enclosing the anonymous commands) 
	 * can be used to define the pointcut.
    */
    pointcut commandExecute(AbstractCommand acommand) :
		this(acommand)
		&& execution(void AbstractCommand+.execute())
		//exclude the anonymous commands 
		&& !within(*..DrawApplication.*)
        && !within(*..CTXWindowMenu.*)
        && !within(*..WindowMenu.*)
        && !within(*..JavaDrawApp.*);

	
	/**
	 * The code checking the reference is crosscutting
	 * and is moved from AbstractCommand.execute() to
	 * this advice. The advice implements now the pre-condition
	 * check for the commands.  
	 */
	before(AbstractCommand acommand) : commandExecute(acommand) {

//		System.out.println("execution : "  
//				+ thisJoinPointStaticPart.getSignature());

		if (acommand.view() == null) {
			throw new JHotDrawRuntimeException("execute should NOT be getting called when view() == null");
		};

	}

	
}
